
#include "cisePort.h"
#include "sim.h"
#include "list.h"
#include "main.h"
#include "route_activity.h"
#include "link.h"
#include "node.h"
/* ------------------- MACRO DEFINITIONS/WRAPPERS -------------- */
#include "link_wrappers.c"

#define MAX_NO_ERROR_TIME (160 * 8 + 40)

extern int status;
int error_happened = 0;

caddr_t
link_action(src,cn,type,pdu,arg)
     Component *src;
     register LINK_ENTITY_TYPE *cn;
     int type;
     PDU_TYPE *pdu;
     caddr_t arg;

{

   caddr_t result = NULL;

   if (cn) 
      dprintf(3 ,"Link layer Event %x  %s %d \n", type,cn->cn_name,cn->cn_class);

   switch (type)  {
      case EV_RECEIVE_START:
         result = cn_receive(cn,src,pdu, 0);
         break;

      case EV_RECEIVE1:
         result = cn_receive(cn,src,pdu, 1);
         break;

      case EV_RECEIVE2:
	 result = cn_receive(cn,src,pdu, 2);
	 break;

      case EV_RECEIVE3:
	 result = cn_receive(cn,src,pdu, 3);
	 break;

      link_other_event_handlers();
   
      default:
         break;
   }

   return(result);
}


/******************************************************************** 

 cn_receive(cn, src, pdu):
   Whenever a link receives any pdu from a node, it just passed the 
   pdu to the next node without any changing. For the interface
   purpose, it also update it status in the interface according to
   it current mode, which could be basic mode (mode == 1), priority
   stack mode (mode == 2), or monitor select mode (mode == 3).

 ********************************************************************/

static caddr_t
cn_receive(cn,src, pdu, start)
    register LINK_ENTITY_TYPE *cn;
    register NODEConnectiont *src;
    PDU_TYPE *pdu;
    int start;
{
  
   register Neighbor *n;
   register NODEConnectiont *dest;
   tick_t tics;
   int pdutype = pdu->type;
   /* static long first_error_time = 0; */
   int mode = (int)cn->cn_mode->u.d;
   int pos, order;

   for (n=(Neighbor *)cn->cn_neighbors->l_head;n;n=n->n_next)
     {
       if (n->n_c != (Component *)src) break;
     }
   dest = (NODEConnectiont *)n->n_c;
   
   tics = USECS_TO_TICKS(cn->cn_delay->u.d);

   if(start==0 || start==1)
     cn->having_data++;
   if(start == 1)
     src->having_data--;

   switch(start)
     {
     case 0:
       ev_enqueue(EV_RECEIVE2, (Component *)src, (Component *)cn, 
		  ev_now()+tics, cn->cn_action, (PDU_TYPE *) pdu, 
		  (caddr_t)NULL);
       break;

     case 1:
       if((mode == 3) &&
	  (cn->cn_name[4] == '1') &&  
	  (ev_now() > MAX_NO_ERROR_TIME * 1000) && 
	  (!error_happened) && 
	  (pdu->type == TOKEN || pdu->type == AMP))
	 {
	   pdu_free(pdu);
	   show_message("Error happened in LINK1. A PDU(%d) is lost at %d\n", 
			pdu->type, ev_now());
	   pdutype = EMPTY;
	   error_happened = TRUE;
	 }
       else 
	 ev_enqueue(EV_RECEIVE2, (Component *)src, (Component *)cn, 
		    ev_now()+tics, cn->cn_action, (PDU_TYPE *) pdu, 
		    (caddr_t)NULL);
       break;

     case 2:
       ev_enqueue(EV_RECEIVE3, (Component *)src, (Component *)cn, 
		  ev_now()+tics, cn->cn_action, (PDU_TYPE *) pdu, 
		  (caddr_t)NULL);
       break;

     case 3:
       ev_enqueue(EV_RECEIVE, (Component *)cn, (Component *)dest, 
		  ev_now()+tics, dest->cn_action, (PDU_TYPE *) pdu, 
		  (caddr_t)NULL);
       break;

     }

 
   if (M.doX) 
     {
       switch (mode) 
	 {
	 case 1: /* Basic mode */
	 case 2: /* Priority stack mode */
	        display_number_in_window((Component *)cn, cn->having_data);
		order = start - 1;
		if (order < 0) order = 0;
		pos = name_to_position(cn->cn_name, order);
		display_token_ring_status(pdu->type, pos);

		if(start) {
	        display_number_in_window((Component *)src, src->having_data);
		pos = name_to_position(src->cn_name, 0);
		if (src->having_data)
		  display_token_ring_status(pdu->type, pos);
		else
		  display_token_ring_status(EMPTY, pos); 
		} 
		break;

	 case 3: /* Monitor select mode */
		order = start - 1;
		if (order < 0) order = 0;
		pos = name_to_position(cn->cn_name, order);
		  display_token_ring_status(pdutype, pos);

		if(!order) 
		  {
		    pos = name_to_position(src->cn_name, 0);
		    display_token_ring_status(EMPTY, pos); 
		  }
		else
		  {
		    pos = name_to_position(cn->cn_name, order-1);
		    display_token_ring_status(EMPTY, pos); 
		  }
		break;	 

	 default:
	 break;
	 }
     }
   

 
   return ((caddr_t)cn);
}
