
/* This is the program for a node action in a TOKEN RING (802.5) Network
 * Programmed for CISE project in Ohio State University  1997/10/10
 */

/* System head files */
#include "cisePort.h"
#include "sim.h"
#include "list.h"
#include "main.h"
#include "route_activity.h"

/* Self defined head files */
#include "stk.h"
#include "node.h"
#include "link.h"

/* interface concerned global variable */
extern Log pkt_log;
extern int doXstuff;

/* Global variables */
int ps, rs;        /* the PPP and RRR value in the frame or token  */
int status = EMPTY;       /* TOKEN, FRAME, EMPTY, AMP, CT and PRG  */
int Having_AMP=FALSE;     /* if the monitor have a queuing AMP     */
char str[80];             /* For temp use                          */
long event_time;          /* For temp use                          */

/* Aliases */
#define NODE_ENTITY_TYPE NODEConnectiont
#define TASKS cn_current->num_of_task

#include "node_wrappers.c"

caddr_t
token_node_action(src, cn, type, pdu, arg)
   Component *src;       /* Source component      */
   NODE_ENTITY_TYPE *cn; /* Destination component */
   int type;
   PDU_TYPE *pdu;
   caddr_t arg;
{

   caddr_t result = NULL;
   PDU_TYPE *newpdu;
   static error_happened = FALSE;

   switch (type)  
     {

     case EV_RECEIVE:
       result = cn_receive(src, cn, pdu);
       break;

     case EV_START:			
       cn_start(cn);						
       result = (caddr_t)cn;
       break;	
					
       /**************************************************************
       ***************************************************************
       
       MAKE SURE YOU UNDERSTAND THE FOLLOWING THREE CASES(For mode 3)

       ***************************************************************
       ***************************************************************/
     case EV_TNT:
       /* when a Timer No Token timeout, if the node is the monitor (This 
	  should always be true, since only monitor deal with TNT), it sends
	  a PRG frame to the ring to clean the ring before it issue a new
	  token */
       show_message("TNT timeout in %s at time %d. Sending PRG frame ...\n",
		    cn->cn_name, event_standard_time);
       if(cn->cn_is_monitor->u.d == 1)  /* this node is the monitor */
	 {
	   /* Set the status to PRG, and sent a purge PDU */
	   pdu_free(pdu);
	   newpdu = (PDU_TYPE *)pdu_alloc();
	   status = PRG;
	   newpdu->type = PRG;
	   cn_send_a_pdu((Component *)cn, 
			 ((Neighbor *) cn->cn_neighbors->l_head)->n_c,
			 0, newpdu);
	 }
       break;
       
     case EV_TSM:
       /* when Timer Standby Monitor timeout, if the node is non-monitor
	  (which should always be true, since only non-monitor nodes 
	  deal with TSM) and the currently status is not PRG (otherwise,
	  some other node(s) TSM timeout first and become the new
	  monitor), send a CT frame to bid for the new monitor 
	  */
       show_message("TSM timeout in %s at time %d. Sending CT frame ...\n", 
		    cn->cn_name, event_standard_time);
       if(cn->cn_is_monitor->u.d != 1 && status != PRG)
	 {
	   pdu_free(pdu);
	   newpdu = (PDU_TYPE *)pdu_alloc();
	   
	   status = CT;
	   newpdu->type = CT;
	   newpdu->snode = cn->cn_name[4];
	   
	   cn_send_a_pdu((Component *)cn, 
			 ((Neighbor *) cn->cn_neighbors->l_head)->n_c,
			 0, newpdu);
	 }
       break;
       
     case EV_AMP:
       if(cn->cn_is_monitor->u.d == 1) 
	 /* Only monitor need to process this event. This include:
	    Send an AMP message to the next node, and
	    Schedule another EV_AMP event(if no error in monitor).
	    */
	 {
	   if((ev_now() >= MONITOR_ERROR_TIME * 1000) && !error_happened)
	     /* make an error in the monitor, so there is no monitor,
		which iccurs EV_TSM to happen
		*/
	     {
	       ev_dequeue_by_comp_and_type((Component *)cn,
					   (Component *)cn,
					   EV_TNT);
	       show_message("An error happened in the monitor(%s) at %d. No monitor present.\n", cn->cn_name, ev_now());
	       cn->cn_is_monitor->u.d = 0;
	       error_happened = TRUE;
	       break;
	     }
	   else
	     {
	       /* before enqueue a new EV_AMP event, dequeue any EV_AMP
		  event from the heap first. This is to make sure
		  only one EV_AMP event is schedules everytime
		  */
               Having_AMP = TRUE;
               show_message("Queuing AMP frame ...(%d)\n", ev_now());
	       ev_dequeue_by_comp_and_type((Component *)cn,
					   (Component *)cn,
					   EV_AMP);
	       event_time = (ev_now()/160000)*160000 + (AMP_VALUE+80)*1000;
	       schedule_an_event(EV_AMP, (Component *)cn, (Component *)cn,
				event_time);
	     }
	 }
       break;
       
       node_other_event_handlers();
       
     default:
       break;
     }
   
   return(result);
}

/*********************************************************************
 *********************************************************************
  cn_start:   Initialize the nodes in the Ring for different mode.
              This includes:
	      -- Create jobs and priorities;
	      -- Generate a new token;
	      -- Start several timers;

	      YOU DONT NEED TO CODE THIS SUBROUTINE, however,
	      YOU DO NEED TO UNDERSTAND THIS SUBROUTINE

 *********************************************************************
 *********************************************************************/
cn_start(cn)
   register NODE_ENTITY_TYPE *cn;
{
  
  char *dest;
  int  i;
  PDU_TYPE *pdu;
  int  mode = cn->cn_mode->u.d;
  
  /*
    if in basic mode or priority mode(mode == 1 or 2), Generate 
    NUM_OF_JOBS destinations for every node, i.e., generate
    NUM_OF_JOBS jobs;
    
    If in priority mode(mode==2), generate priorities for these jobs;
    
    Initialize the node;
    
    If the node is the monitor, generate a token and send it to its
    neighbor link;
    
    If the mode is mode 3 (monitor management):
    -- For non-monitor node, start a TSM timer;
    -- For monitor node, start a TNT timer, issue an AMP message,
       and send out a token
    */
  
  if(mode == 1 || mode == 2)
    {
      /* generate jobs */
      for(i=0; i<NUM_OF_JOBS; i++) 
	while((cn->jobs_dest[i] = rand()%4+1) == 
	      nodename_to_number(cn->cn_name))
	  ;  /* make sure the destination is not itself */
      
      dprintf(1, "%s job destinations:", cn->cn_name);
      for(i=0; i<NUM_OF_JOBS; i++)
      {
	dprintf(1, "%d ", cn->jobs_dest[i]);
	push_dest_element((Component *)cn, cn->jobs_dest[i]);
      }
      dprintf(1, "\n");
      fflush(stdout);
    }
  
  if(mode == 2) /* Priority mode */
    {
      /* Generate priorities for the jobs */
      for(i=0; i<NUM_OF_JOBS; i++)
	cn->jobs_priority[i] = rand() % NUM_OF_PRIORITIES;
      
      dprintf(1, "%s job priorities  :", cn->cn_name);
      for(i=0; i<NUM_OF_JOBS; i++)
      {
	dprintf(1, "%d ", cn->jobs_priority[i]);
	push_pri_element((Component *)cn, cn->jobs_priority[i]);
      }
      dprintf(1, "\n");
      fflush(stdout);
      
      /* Initialize two stacks for every node, which will be used to 
	 save the priority values */
      stkInit(&cn->Sx);
      stkInit(&cn->Sr);
    }
  
  /* Initialize this node */
  cn->bit_counter = 0;
  cn->having_data = 0;
  cn->current_byte_type = NONE;
  cn->is_source = FALSE;
  cn->num_of_task = NUM_OF_JOBS;
  
  if((mode != 3) && (cn->cn_is_monitor->u.d == 1)) 
    /* Monitor in mode 1 or mode 2 */
    {
      show_who_is_monitor(cn->cn_name); 
      cn_send_token((Component *)cn,
		    ((Neighbor *) cn->cn_neighbors->l_head)->n_c,
		    0, 0, 0x10, cn->cn_mode->u.d);
      status = TOKEN;	
      rs = ps = 0;
      show_info("ppp", "0");
      show_info("rrr", "0");
      show_info("monitor", cn->cn_name);
      show_info("src", "NONE");
      show_info("dest", "NONE");
      show_message("Current mode is %d\n", mode);
    }
  
  if (mode == 3)
    {
      if (cn->cn_is_monitor->u.d == 1)
	/* Monitor in mode 3 */
	{
	  show_who_is_monitor(cn->cn_name);  
	  show_info("ppp", "N/A");
	  show_info("rrr", "N/A");
	  show_info("monitor", cn->cn_name);
	  show_info("src", "N/A");
	  show_info("dest", "N/A");
	  show_message("Current mode is %d\n", mode);
	  
	  /* Prepare and send out a token */
	  pdu = (PDU_TYPE *)pdu_alloc();
	  pdu->type = TOKEN;
	  status = TOKEN;
	  cn_send_a_pdu((Component *)cn,
			((Neighbor *) cn->cn_neighbors->l_head)->n_c,
			0, pdu);
	  /* Schedule an EV_AMP event */
	  event_time = ev_now() + (AMP_VALUE+80)*1000;
	  schedule_an_event(EV_AMP, (Component *)cn, (Component *)cn,
			   event_time);
	  /* Start the TNT timer */
	  event_time = ev_now() + TNT_VALUE * 1000;
	  schedule_an_event(EV_TNT, (Component *)cn, (Component *)cn,
		     event_time);
	}
      else  
	/* Non-monitor nodes node in mode 3 */
	{
	  /* Start the TSM timer */
	  event_time = ev_now() + TSM_VALUE * 1000;
	  schedule_an_event(EV_TSM, (Component *)cn, (Component *)cn,
			    event_time);
	}	   
    }
  
  return (1);
}



/*****************************************************************************
 *****************************************************************************
  cn_receive: This is the most important of this lab. Whenever a bit (mode1
              and mode 2) or a frame (mode3) is received from a pdu, 
	      the node should do every kinds of check, copying, and 
	      state updating;

	      YOU DONT NEED TO CODE THIS SUBROUTINE, however,
	      YOU DO NEED TO UNDERSTAND THIS SUBROUTINE

 *****************************************************************************
 *****************************************************************************/
static caddr_t
cn_receive(cn_src,cn_current, pdu)
     register NODE_ENTITY_TYPE *cn_current; /* cn_current is the receiver */
     LINKConnectiont *cn_src;               /* cn_src is the sender       */
     PDU_TYPE *pdu;
{
  register Neighbor *cn_tmp;
  register LINKConnectiont *cn_dest;
  int mode = (int)cn_current->cn_mode->u.d;
  tick_t tics;
  
  /* Find the next neighbor of cn_current -- cn_dest. 
     Normally, almost all the received pdus should be resend to this cn_dest 
     */
  for (cn_tmp=(Neighbor *)cn_current->cn_neighbors->l_head;
       cn_tmp;
       cn_tmp=cn_tmp->n_next)
    {
      if (cn_tmp->n_c != (Component *)cn_src) break;
    }
  cn_dest = (LINKConnectiont *)cn_tmp->n_c;
  


  /* deal with monitor select mode -- mode3 */
  if(mode == 3)
    {
      return (cn_receive_in_mode3(cn_src, cn_current, cn_dest, pdu));
    }
  


  /* Initialization for mode1 and mode2 */
  /************************************************************************/
  /* Update the counter for source and current component */
  cn_current->having_data++;
  cn_src->having_data--;
  
  /* The monitor contains a 24-bit buffer, while a normal node contains
     only a 4-bit buffer;
     Normally when we talk about 1-bit delay, the non-monitor node 
     should contains only one bit. However, in this lab, since the number 
     of node is small and the link length is not long enough to contain
     lots of bits, we set 4 bits for every non-monitor node, which makes
     the interface looks better */
  tics = USECS_TO_TICKS(cn_current->cn_delay->u.d);
  if (cn_current->cn_is_monitor->u.d == 1)
    tics *= 24;
  else
    tics *= 4;
  
  /* Since current node received a new bit, the bit counter should 
     increased by 1 */
  cn_current->bit_counter++;
  
  /* when the last bit of a token is received, reset the bit counter 
     and byte_type */
  if(cn_current->bit_counter == 8 && cn_current->current_byte_type == FC &&
     pdu->type == TOKEN)
    {
      cn_current->current_byte_type = NONE;
      cn_current->bit_counter = 0;
    }
  
  /* Whenever 8 bits are received, update the byte_type;
     When a frame or a token is complete, reset the byte_type; */
  if(cn_current->bit_counter == 9 || cn_current->bit_counter == 1)
    {
      cn_current->bit_counter = 1;
      cn_current->current_byte_type++;
      
      if((pdu->type == TOKEN && cn_current->current_byte_type > FC) ||
	 (pdu->type == FRAME && cn_current->current_byte_type > FS))
	cn_current->current_byte_type = SD;
    }

  /************************************************************************/
  /* End of initialization */
  


  /* Deal with mode 1 and mode2 */

  if(mode == 1) /* basic mode -- one bit delay */
    { 
      return(cn_receive_in_mode1(cn_src, cn_current, cn_dest, pdu, tics));
    }
  
  if (mode == 2) /* mode 2 -- Priority Stack mode */
    {
      return(cn_receive_in_mode2(cn_src, cn_current, cn_dest, pdu, tics));
    } 
  
  return(NULL);
}

/************************************************************************
 ************************************************************************
 ************************************************************************

                     YOUR CODE BEGINS HERE!!! 

 ************************************************************************
 ************************************************************************
 ************************************************************************/


/* Deal with received pdus when the current mode is mode 1 */
static caddr_t
cn_receive_in_mode1(cn_src, cn_current, cn_dest, pdu, tics)
     register NODE_ENTITY_TYPE *cn_current;
     LINKConnectiont *cn_src, *cn_dest;
     PDU_TYPE *pdu;
     tick_t   tics;
{
  int node_src, node_dest;
  int pos;
  int pdutype = pdu->type;
  
  if((status == TOKEN) && (cn_current->num_of_task > 0) &&
     (cn_current->current_byte_type==SD)&&(cn_current->bit_counter==1))
    /* i.e.:
       1: This is a token;
       2: current node has job to send;
       3: this is the first bit of the token;
       Normally, you don't know if this is a token or a frame before 
       you receive the T bit in the Access Control field, which is the 
       12th bit of a token. To simplify it, we put this bit in every 
       pdu, so we know this is a token in advance; 
       All the process about reserve and get a token is done in this
       first bit of a token; all the rest bits of a token is not 
       considered; */
    {
      /* Current node get the token and prepare to send a frame */
      node_src = nodename_to_number(cn_current->cn_name);
      node_dest = cn_current->jobs_dest[cn_current->num_of_task-1];
      
      show_info("src", cn_current->cn_name);
      sprintf(str, "node%d", node_dest);
      show_info("dest", str);
      show_message("%s is sending a frame to %s\n",
		   cn_current->cn_name, str);
      
      /********** Code place 1.1: **********
	 1: update status;
	 2: set is_source variable for cn_current;
	 */



      pdu_free(pdu);
      cn_current->having_data--;
      
      /* ps=0, rs=0, ac=0x10(token bit is set), fc=0x00, fs=0 
	 In this lab, the exact values of these fields are not
	 important, since we don't extract them out. For simplicity,
	 all the needed information is contained in the pdu
	 Only SD, AC, FC, DA, SA, DATA, ED and FS are concerned.
	 And every field contain only one byte. */
      
      /********** Code place 1.2: **********
	1: send a frame to cn_dest by cn_current;
	2: Since we've finished sending a job, 
	the num_of_task should be decreased by 1; 
	*/




      pop_dest_element((Component *)cn_current);
    }
  else if((status == TOKEN) && (cn_current->num_of_task == 0))
    /* i.e.: this is a token but cn_current has no job to send
       Just pass it to the next node */
    {
      /********** Code place 1.3 **********
	Pass pdu to its destination (at time tics)
	*/


    }
  else if(status == FRAME)
    /* This is a frame */
    {
      if(pdu->type == TOKEN)
	/* Since when a node gets the first bit of a token and sets the 
	   status to FRAME, the sending of a token is not finished. So 
	   even in FRAME status, there are pdus whose type are TOKEN.
	   Since we don't need these bit any more, just ignore it and
	   free the pdu space; */
	{
	  pdu_free(pdu);
	  cn_current->having_data--;
	}
      else if (cn_current->is_source)
	/* i.e.: cn_current is the source who sent this frame.
	   So this is a returning bit; */
	{
	  /* free the pdu and update having_data counter*/
	  pdu_free(pdu);
	  cn_current->having_data--;
	  /* if this is the last bit of the returning frame, generate
	     a new token and send it to the next node; */
	  if(cn_current->bit_counter == 8 &&
	     cn_current->current_byte_type == FS)
	    {
	      /********** Code place 1.4 **********
		1: reset status;
		2: update is_source for cn_current;
		3: send a new token to dest
		*/




	      show_message("Sending new token ...\n");
	      cn_current->current_byte_type = NONE;
	      /* the next two lines is used for display */
	      pdutype = EMPTY;
	      cn_current->having_data = 0;
	    }
	}
      else 
	/* cn_current is not the source
	   Just pass this pdu to the next node */
	{
	  /********** Code place 1.5 **********
	    pass pdu to next component at time tics
	    */
	  


	  if ((pdu->dnode == nodename_to_number(cn_current->cn_name)) &&
	      (cn_current->current_byte_type == DATA) &&
	      (cn_current->bit_counter == 8))
	    /* cn_current is the destination. 
	       show a message to claim that a frame has been received; */
	    {
	      show_message("A frame received by %s from node%d\n",
			   cn_current->cn_name, pdu->snode);
	    } 
	}
    }   
  
  if (M.doX)
    /* Show the new status of the token ring. 
       You can ignore this part */ 
    {
      display_number_in_window((Component *)cn_current, 
			       cn_current->having_data);
      pos = name_to_position(cn_current->cn_name, 0);
      display_token_ring_status(pdutype, pos);
      
      display_number_in_window((Component *)cn_src, 
			       cn_src->having_data);
      switch(cn_src->having_data)
	{
	case 0:
	  display_token_ring_status(EMPTY, 
				    name_to_position(cn_src->cn_name, 2));
	case 1:
	  display_token_ring_status(EMPTY, 
				    name_to_position(cn_src->cn_name, 1));
	case 2:
	  display_token_ring_status(EMPTY, 
				    name_to_position(cn_src->cn_name, 0));
	default:
	  break;
	}
      
    }
  return ((caddr_t)cn_current);
}



/* Deal with received pdus when the current mode is mode 2 */
static caddr_t
cn_receive_in_mode2(cn_src, cn_current, cn_dest, pdu, tics)
     register NODE_ENTITY_TYPE *cn_current;
     LINKConnectiont *cn_src, *cn_dest;
     PDU_TYPE *pdu;
     tick_t   tics;
{
  int node_src, node_dest;
  int top, pos;
  int pdutype = pdu->type;
  
  if(status == TOKEN)
    {
      if(cn_current->current_byte_type == SD && 
	 cn_current->bit_counter == 1) 
	/* This is the first bit of the token, and is also the only place 
	   to deal with priority. Only the rs_value and ps_value of this 
	   pdu are useful -- Just for simplicity   */
	{   
	  /* extract the ps and rs from the pdu */
	  ps = pdu->ps_value;
	  rs = pdu->rs_value;
	  /* Display the ps and rs value */
	  sprintf(str, "%d", ps);
	  show_info("ppp", str);
	  sprintf(str, "%d", rs);
	  show_info("rrr", str);
	  
	  /* case 1: Lower the ps value */
	  if(!stkTop(&(cn_current->Sx), &top))
	    top = -1;
	  if(ps > rs && ps == top)
	    /* Since this ps equals to my saved value, it was I who
	       increase this ps value before, therefore, it's my 
	       responsibility to lower it down */
	    {
	      stkPop(&(cn_current->Sx), &top);
	      if(!stkTop(&(cn_current->Sr), &top))
		top = -1;
	      /* If the the current rs value is greater than the saved 
		 old ps, we can't just set the new ps to the old ps.
		 We should set the new ps to rs and save this new
		 ps (remember, it is I again who increase the ps from 
		 the old ps, which is lower than rs, to a new ps (which
		 equals to rs). So, I should save it in order to lower 
		 it down in the future) */
	      if(rs > top)
		{
		  /********** Code place 2.1 **********/


		}
	      else
		/* otherwise, set the new ps to the old saved ps */
		{
		  /********** Code place 2.2 **********/



		}
	      show_message("Ps is lowered down by %s\n", 
			   cn_current->cn_name);
	    }
	  
	  if(TASKS)  
	    /* I have frames to send */
	    {
	      if(cn_current->jobs_priority[TASKS-1] >= ps)
		/* case 2: cn_current has a higher priority job to send,
		   so get the current token */
		{
		  /********** Code place 2.3 **********/
		  /* set the status to FRAME to get this token 
		     and reset rs */
		  

		  node_src = nodename_to_number(cn_current->cn_name);
		  node_dest = cn_current->jobs_dest[TASKS-1];
		  cn_current->is_source = TRUE;
		  
		  show_info("src", cn_current->cn_name);
		  sprintf(str, "node%d", node_dest);
		  show_info("dest", str);
		  show_message("%s gets the token. Sending a frame to %s\n", 
			       cn_current->cn_name, str);
		  
		  pdu_free(pdu);
		  cn_current->having_data--;
		  
		  /********** Code place 2.4 **********
		    1: send a frame to cn_dest by cn_current;
		    2: update num_of_task;
		    */



		  pop_pri_element((Component *)cn_current);
		}
	      else 
		/* the job priority of cn_current is not high enough
		   to get the token */
		{
		  /* case 3: reserve the next token */
		  if(cn_current->jobs_priority[TASKS-1] > rs)
		  /********** Code place 2.5 **********/


		  
		  /* Pass the pdu to the next node */
		  pdu->rs_value = rs;
		  pdu->ps_value = ps;
		  cn_send_a_pdu((Component *)cn_current,
				(Component *)cn_dest, 
				tics, pdu);
		}
	    } /* if(TASKS) */
	  else 
	    /* I have no frame to send, so just pass it to the next node */
	    {
		  /********** Code place 2.6 **********/



	    }
	}
      else 
	/* The rest bits of a Token. Need no special action.
	   Just pass it to the next node */
	{
	  cn_send_a_pdu((Component *)cn_current,
			(Component *)cn_dest, 
			tics, pdu);
	}
    }
  /* The above deals with TOKEN bits, while the follow deals with
     the FRAME bits */
  else  
    /* this is a frame bit */
    {
      if(pdu->type == TOKEN)
	/* the same as mode 1. See the former comments */
	{
	  if(cn_current->is_source)
	    {
	      pdu_free(pdu);
	      cn_current->having_data--;
	    }
	  else
	    {
	      cn_send_a_pdu((Component *)cn_current,
			    (Component *)cn_dest, 
			    tics, pdu);
	    }
	}		 
      else 
	/* normal bit from a frame -- not from a token */
	{
	  if(cn_current->current_byte_type == SD && 
	     cn_current->bit_counter == 1) 
	    /* the first bit of a frame. Do all the ps/rs recording or 
	       updating here */
	    {
	      ps = pdu->ps_value;
	      rs = pdu->rs_value;
	      
	      if(cn_current->is_source)
		/* cn_current is the source who sent this bit before.
		   So this is the first returning bit. Just free its
		   space */
		{
		  pdu_free(pdu);
		  cn_current->having_data--;
		}
	      else
		/* cn_current is not the source
		   if possible, reserve the next token by setting the 
		   rs value in the pdu;
		   pass this pdu to the next node; */
		{
		  if(TASKS && cn_current->jobs_priority[TASKS -1] > rs)
		  /********** Code place 2.7 **********/



		  
		  pdu->rs_value = rs;
		  pdu->ps_value = ps;
		  cn_send_a_pdu((Component *)cn_current,
				(Component *)cn_dest, 
				tics, pdu);
		}
	    }
	  else 
	    /* the rest bits of a frame -- not the first bit */
	    {
	      if(cn_current->is_source)
		{
		  /* Free the pdu space */
		  pdu_free(pdu);
		  cn_current->having_data--;
		  
		  if(cn_current->bit_counter == 8 &&
		     cn_current->current_byte_type == FS)
		    /* the last bit returns to the source 
		       Set status back to TOKEN;
		       Reset the current_byte_type and is_source fields;
		       Generate a new TOKEN according to the proper ps
		       and rs values */
		    {
		      status = TOKEN;
		      cn_current->current_byte_type = NONE;
		      cn_current->is_source = FALSE;
		      
		      if(rs > ps)
			/* if rs > ps, generate a token with ps=rs and
			   rs=0, save old ps and rs the stack for the 
			   future lower down operation */
			{
			  /********** Code place 2.8 **********/



			  show_message("Ps is increased by %s\n",
				       cn_current->cn_name);
			} /* if(rs > ps) */
		      
		      else if(rs < ps)
			{
			  /* case 1: Lower the ps value */
			  if(!stkTop(&(cn_current->Sx), &top))
			    top = -1;
			  if(ps == top)
			    /* Since this ps equals to my saved value, 
			       it was I who increase this ps value before,
			       therefore, it's my responsibility to lower
			       it down 
			       The same as status==TOKEN case. */
			    {
			      stkPop(&(cn_current->Sx), &top);
			      if(!stkTop(&(cn_current->Sr), &top))
				top = -1;
			      if(rs > top)

				{
				  /********** Code place 2.9 **********/



				}
			      else
				{
				  /********** Code place 2.10 **********/




				}
			      show_message("Ps is lowered down by %s\n",
					   cn_current->cn_name);
			    }
			}
		      else /* ps == rs */
			rs = 0;
		      
		      /* generate a new token according to the ps and rs
			 got by the above calculation;
			 send this token to the next node; */
		      pdu->ps_value = ps;
		      pdu->rs_value = rs;			   
		      cn_send_token(cn_current, cn_dest, ps, rs, 
				    0x10, 2);
		      
		      show_message("Sending new token ...\n");
		      show_info("src", "NONE");
		      show_info("dest", "NONE");
		      
		      /* the next two lines is used to display purpose */
		      pdutype = EMPTY;
		      cn_current->having_data = 0;
		    } /* if last bit returns */
		} /* if(cn_current->is_source) */
	      else 
		/* cn_current is not the source, i.e., this is not a 
		   returning bit, so just pass it to the next node; */
		{
		  cn_send_a_pdu((Component *)cn_current,
				(Component *)cn_dest, 
				tics, pdu);
		  
		  if((pdu->dnode==nodename_to_number(cn_current->cn_name))
		     && (cn_current->current_byte_type == DATA)
		     && (cn_current->bit_counter == 8))
		    /* cn_current is the destination, show this */
		    {
		      show_message("A frame received by %s from node %d\n",
				   cn_current->cn_name, pdu->snode);
		    } /* if destination */
		} /* else -- non-source */
	    } /* else -- the rest bits */
	} /* else -- normal bit from a frame */
    } /* else -- this is a frame */
  
  if (M.doX)
    /* Show the new status of the token ring. 
       Please ignore it  */
    {
      display_number_in_window((Component *)cn_current, 
			       cn_current->having_data);
      pos = name_to_position(cn_current->cn_name, 0);
      display_token_ring_status(pdutype, pos);
      
      display_number_in_window((Component *)cn_src, 
			       cn_src->having_data);
      switch(cn_src->having_data)
	{
	case 0:
	  display_token_ring_status(EMPTY, 
				    name_to_position(cn_src->cn_name, 2));
	case 1:
	  display_token_ring_status(EMPTY, 
				    name_to_position(cn_src->cn_name, 1));
	case 2:
	  display_token_ring_status(EMPTY, 
				    name_to_position(cn_src->cn_name, 0));
	default:
	  break;
	}
    }
  return ((caddr_t)cn_current);
}



/* Deal with received pdus when the current mode is mode 3 */
static caddr_t
cn_receive_in_mode3(cn_src, cn_current, cn_dest, pdu)
     register NODE_ENTITY_TYPE *cn_current;
     LINKConnectiont *cn_src, *cn_dest;
     PDU_TYPE *pdu;
{
  PDU_TYPE *newpdu;
  int pdutype = pdu->type;
  int pos;
  tick_t tics;
  tick_t event_time; /* IMPORTANT: Used for setting the time parameter of
			schedule_an_event. 
			Please DO NOT CHANGE IT and DO USE IT
			*/
  
  tics = USECS_TO_TICKS(cn_current->cn_delay->u.d);
  switch(status)
    {
    case TOKEN:
      if(pdutype != TOKEN)
	{
	  pdu_free(pdu);
	  break;
	}
      if(cn_current->cn_is_monitor->u.d == 1) 
	{
	  if(Having_AMP)
	    /* there is a queued AMP waiting for a token */
	    {
	      pdu_free(pdu);
	      newpdu = (PDU_TYPE *)pdu_alloc();
	      status = AMP;
	      newpdu->type = AMP;
	      Having_AMP = FALSE;
	      show_message("Sending AMP frame ...(%d)\n", ev_now());
	      cn_send_a_pdu((Component *)cn_current, 
			    (Component *)cn_dest,
			    tics, newpdu);
	    }
	  else
	    {
	      cn_send_a_pdu((Component *)cn_current,
			    (Component *)cn_dest, 
			    tics, pdu);
	    }
	  /* This is the monitor. Since it sees a token,
	     it should reset its TNT timer. This is done
	     by dequeue the old one and schedule a new one.
	     */
	  ev_dequeue_by_comp_and_type((Component *)cn_current,
				      (Component *)cn_current,
				      EV_TNT);
	  event_time = ev_now() + TNT_VALUE * 1000;
	  /********** Code place 3.1 **********/
	  
	  
	}
      else
	{
	  /* pass the token to next node */
	  cn_send_a_pdu((Component *)cn_current,
			(Component *)cn_dest, 
			tics, pdu);
	}
      break;
      
    case PRG:
      if(cn_current->cn_is_monitor->u.d != 1)
	/* non-monitor node. Pass the pdu to next node
	   and refresh the TSM timer */
	{
	  cn_send_a_pdu((Component *)cn_current,
			(Component *)cn_dest, 
			tics, pdu);
	  ev_dequeue_by_comp_and_type((Component *)cn_current,
				      (Component *)cn_current,
				      EV_TSM);
	  event_time = ev_now() + TSM_VALUE * 1000;
	  /********** Code place 3.2 **********/



	}
      else  
	/* current node is the monitor */
	{
	  if (pdu->type == PRG)
	    /* The PRG frame issued by this node before comes back.
	       The ring is clean now and it is ready to issue a
	       new token */
	    {
	      /* Issue a new token */
	      newpdu = (PDU_TYPE *)pdu_alloc();
	      newpdu->type = TOKEN;
	      show_message("Purge finished. Sending new Token ...(%d)\n",
			   ev_now());
	      /********** Code place 3.3 **********
		1: update the status;
		2: send out newpdu (at time 0)
		*/
		



	      /* Since this is probably a newly selected monitor
		 it need to start the TNT timer and schedule a 
		 AMP message */
	      /********** Code place 3.4 **********/
	      event_time = ev_now() + TNT_VALUE * 1000,


	      event_time = ev_now() + (AMP_VALUE+80)*1000;



	    }
	  /* purge every pdu */
	  pdutype = EMPTY;
	  pdu_free(pdu);
	}
      break;
      
    case CT:
      if(pdu->snode != cn_current->cn_name[4])
	/* the source address doesn't match current node's address,
	   just pass it to the next node */
	{
	  cn_send_a_pdu((Component *)cn_current,
			(Component *)cn_dest, 
			tics, pdu);
	}
      else 
	/* The source address matches current node's address, 
	   so the CT comes back in CT state. This is to say,
	   I am the new monitor now */
	{
	  pdu_free(pdu);
	  if(status == CT)
	    {
	      /* Set me as the monitor */
	      /********** Code place 3.5 **********/



	      /* dequeue the TSM, which is a timer for non-monitor
		 node */
	      ev_dequeue_by_comp_and_type((Component *)cn_current,
					  (Component *)cn_current,
					  EV_TSM);
	      /* Prepare and send out a PRG frame to clear
		 the ring before it issue a new token */
	      newpdu = (PDU_TYPE *)pdu_alloc();
	      newpdu->type = PRG;
	      show_who_is_monitor(cn_current->cn_name);
	      show_info("monitor", cn_current->cn_name);
	      show_message("%s is the new monitor. Purging ...(%d)\n",
			   cn_current->cn_name, ev_now());
	      /********** Code place 3.6 **********
		1: update status;
		2: send out newpdu (at time tics)
		*/



	    }
	}
      break;
      
    case AMP:
      if(pdutype == AMP) 
	{
	  if(cn_current->cn_is_monitor->u.d != 1) 
	    /* When a non-monitor node sees a AMP, it reset its
	       TSM timer and passes AMP to the next node */
	    {
	      /* dequeue the old TSM */
	      ev_dequeue_by_comp_and_type((Component *)cn_current,
					  (Component *)cn_current,
					  EV_TSM);
	      /* enqueue a new TSM */
	      event_time = ev_now() + TSM_VALUE * 1000;
	      /********** Code place 3.8 **********/



	      /* pass this AMP to next node */
	      cn_send_a_pdu((Component *)cn_current,
			    (Component *)cn_dest, 
			    tics, pdu);
	    }
	  else  
	    /* Monitor, who sent this AMP message before.
	       Just free it and set the status back to token
	       Since this AMP means a former token, so restore 
	       the token and reschedule the TNT is needed also */
	    {
	      pdu_free(pdu);
	      newpdu = (PDU_TYPE *)pdu_alloc();
	      newpdu->type = TOKEN;
	      status = TOKEN;
	      cn_send_a_pdu((Component *)cn_current,
			    (Component *)cn_dest,
			    tics, newpdu);
	      ev_dequeue_by_comp_and_type((Component *)cn_current,
					  (Component *)cn_current,
					  EV_TNT);
	      event_time = ev_now() + TNT_VALUE * 1000;
	      schedule_an_event(EV_TNT, (Component *)cn_current,
				(Component *)cn_current, event_time);
	    }
	}
      break;
      
    default:
      printf("should not be here (below AMP)\n");
      break;
    }
  
  
  /* Update the interface according to current pdu type 
     You can ignore this part */
  if(M.doX) 
    {
      pos = name_to_position(cn_current->cn_name, 0);
      display_token_ring_status(pdutype, pos);
      display_token_ring_status(EMPTY, name_to_position(cn_src->cn_name, 2));
    }
  
  return((caddr_t)cn_current);
}

