

#ifndef NODE_H
#define NODE_H

#include "q.h"
#include "eventdefs.h"
#include "component.h"
#include "log.h"
#include "comptypes.h"
#include "pdu.h"
#include "event.h"
#include "route.h"
#include "subr.h"
#include "sim_tk.h"
#include "stk.h"

#define NUM_OF_JOBS 3
#define NUM_OF_PRIORITIES 8

#define TNT_VALUE	(160 * 2)
#define TSM_VALUE	(160 * 4)
#define AMP_VALUE	(160 * 2)
#define MONITOR_ERROR_TIME 	(160 * 16 + 80)

#define START_OF_FRAME 0xFF
#define END_OF_FRAME   0x00

#define TOKEN 0
#define FRAME 1
#define EMPTY 2
#define AMP   3  /* Active Monitor Present */
#define CT    4  /* Claim Token Frame */
#define PRG   5  /* Purge frame */

enum byte_type {NONE=0, SD, AC, FC, DA, SA, DATA, ED, FS};

typedef struct _NODEConnectiont{
  struct _NODEConnectiont *cn_next, *cn_prev;
  short         cn_class;
  short         cn_type;
  char          cn_name[40];    /* Name of component (appears on screen) */
  PFP           cn_action;      /* Main function of component.  */
  XComponent    *cn_picture;    /* Graphics object that displays this thing */
  list          *cn_neighbors;  /* List of neighbors of this thing */
  
  /* Parameters-- data that will be displayed on screen */
  short         cn_menu_up;     /* If true, then the text window is up */
  short         cn_big_menu;    /* not used */
  queue         *cn_params;     /* Variable-length queue of parameters */
  
  int           cn_iq_length;   /* get the current input q length here */
  int           cn_iq_limit;    /* get the current input q length here */
  
  
  int           cn_flash;
  
  /* Parameters -- data that will be logged  */
  Param         *cn_Name;
  Param         *cn_mode;
  Param         *cn_is_monitor;
  Param         *cn_having_error;
  Param         *cn_delay;

  /* assistant variables of this component */
  int           bit_counter;   /* counter for collecting bits to form a byte*/
  int           having_data;
  enum byte_type  current_byte_type;     /* the type of  most recent 8 bits,
					  used for batch transmission */
  int           is_source;      /* I am the source */
  int           jobs_priority[NUM_OF_JOBS]; /* job priorities */
  int           jobs_dest[NUM_OF_JOBS];     /* job destinations */
  Stk           Sx, Sr;         /* the priority stacks  */
  int           num_of_task;    /* number of valid tasks in the task set */
  
} NODEConnectiont;

#define MY_SEND_A_PDU (Evtype)(EV_CLASS_PRIVATE | 1)

caddr_t token_node_action(Component *src, NODEConnectiont *node, int type, PDU_TYPE *pdu, caddr_t arg);


int nodename_to_number(char *);
void number_to_nodename(int, char *);
int name_to_position(char *, int);
void show_message(char *, ...);

#endif
  
