/* TOKEN RING NODE WRAPPER ROUTINES */

static caddr_t cn_reset(),cn_create(),cn_neighbor(),cn_receive(),
               cn_uneighbor(),cn_route(),cn_hops(),cn_send_token(),
               cn_send_frame(),cn_send_a_byte(),cn_send_a_pdu(),
	       cn_receive_in_mode1(),cn_receive_in_mode2(),
	       cn_receive_in_mode3();

int   cn_start();


#define node_other_event_handlers()				\
{								\
     case EV_RESET:						\
        result=cn_reset(cn);					\
        break;							\
     case EV_CREATE:						\
        result = cn_create((char *)arg);			\
        break; 							\
     case EV_DEL:						\
        comp_delete((Component *)cn);				\
        result = (caddr_t)cn;					\
        break;							\
     case EV_NEIGHBOR:						\
        result = cn_neighbor(cn, (Component *)arg);		\
        break;							\
     case EV_UNEIGHBOR:						\
        result = cn_uneighbor(cn, (Component *)arg);		\
        break;							\
     case EV_LEGAL_NEXT_HOPS:					\
        result = cn_hops(cn);					\
        break;							\
     case EV_MAKE_ROUTE:					\
        result = (caddr_t) cn_route(cn, (list *)arg);		\
        break;							\
     case EV_UPDATE_ROUTE_ACTIVITY:				\
        update_routes(src,(Component *)cn);              	\
        result=(caddr_t)cn;					\
        break;                                           	\
}								\



/**************************************************************
  cn_send_a_pdu: Send a pdu to cn_dest by cn_src in mode 3;
 **************************************************************/
static caddr_t cn_send_a_pdu(cn_src, cn_dest, tics, pdu)
   register NODE_ENTITY_TYPE *cn_src;
   register LINKConnectiont *cn_dest;
   tick_t   tics;
   PDU_TYPE *pdu;
 {
   ev_enqueue(EV_RECEIVE1, (Component *)cn_src,
	      (Component *)cn_dest, ev_now()+tics,
	      cn_dest->cn_action, (PDU_TYPE *)pdu, (caddr_t)NULL);
   return((caddr_t)cn_src);
 }


/**************************************************************
  cn_send_a_byte: Send a byte bit by bit to cn_dest by cn_src;
 **************************************************************/
static caddr_t cn_send_a_byte(cn_src, cn_dest, src, dest, ps_value, rs_value, 
			      bytevalue, mode, delay, stat)
   register NODE_ENTITY_TYPE *cn_src;
   register LINKConnectiont *cn_dest;
   int      src, dest;
   int      ps_value, rs_value;
   unsigned char bytevalue;
   int      mode;
   int      delay;
   int      stat; /* status, i.e., TOKEN or FRAME */
 {
   /*
      It is the sender's responsibility to allocate space for the pdu 
      in which it uses to send the data, so allocate and prepare a pdu 
      for every bit 
      */

   int i;
   tick_t tics;
   PDU_TYPE *pdu;

   tics = USECS_TO_TICKS(cn_src->cn_delay->u.d);

   for (i=0; i<8; i++)
     {
       /* Allocate space and initialize a pdu */
       pdu = (PDU_TYPE *)pdu_alloc();
       pdu->snode = src;
       pdu->dnode = dest;
       pdu->data = bytevalue & (0x01 << i);
       pdu->type = stat;
       pdu->rs_value = rs_value;
       pdu->ps_value = ps_value;
       /* Send this pdu to its neighbor link */
       ev_enqueue(EV_RECEIVE_START, (Component *)cn_src,
		  (Component *)cn_dest, 
	          ev_now()+(delay*8 + i+1)*tics,
		  cn_dest->cn_action, (PDU_TYPE *)pdu, (caddr_t)NULL);
     }

   return ((caddr_t)src);
 }

/******************************************************************
  cn_send_token: send a token to cn_dest by cn_src;
                 This includes sending the three bytes of a token;
		 So the only thing to do is to call cn_send_a_byte
		 three times;
 ******************************************************************/
static caddr_t cn_send_token(cn_src, cn_dest, ps_value, rs_value, ac, mode)
   register NODE_ENTITY_TYPE *cn_src;
   register Component *cn_dest;
   int      ps_value, rs_value;
   unsigned char     ac; /* Access Control field */
   int      mode;
{
  /* send the three bytes of a token; 
     A token need not to know the sender and has no destionation,
     so, -1 is used as the src and dest
     */
  cn_send_a_byte(cn_src, cn_dest, -1, -1, ps_value, rs_value, 
		 START_OF_FRAME, mode, 0, TOKEN);
  cn_send_a_byte(cn_src, cn_dest, -1, -1, ps_value, rs_value, 
		 ac, mode, 1, TOKEN);
  cn_send_a_byte(cn_src, cn_dest, -1, -1, ps_value, rs_value, 
		 END_OF_FRAME, mode, 2, TOKEN);

  return ((caddr_t)cn_src);
}

/********************************************************************
  cn_send_frame: send a frame to cn_dest by cn_src, i.e., sending
                 8 bytes
 ********************************************************************/
static caddr_t cn_send_frame(cn_src, cn_dest, ps_value, rs_value, ac, fc, da, sa, data, fs, mode)
   register NODE_ENTITY_TYPE *cn_src;
   register Component *cn_dest;
   int      ps_value, rs_value;
   unsigned char ac, fc, da, sa, data, fs;
   int      mode;
{
  /* Prepare the bytes of a frame: SD, AC, FC, DA, SA, DATA, ED, and FS
     and send them out;
     
     Normally, a complete frame contains Starting Delimiter (SD), Access 
     Control (AC), Frame Control (FC), Destination Address (DA, 2 - 6 bytes),
     Source Address (SA, 2 - 6 bytes), Data Unit (>= 0 bytes), Frame-check 
     Sequence (FCS), Ending Delimiter (ED) and Frame Status (FS). In order 
     to simplify the trival parts, we only choose SD, AC, FC, DA, SA, DATA,
     and FS, and DA, SA, DATA all contain only one byte. FCS is not considerd 
     here. So totally 8 bytes needed to send here.
     */

  cn_send_a_byte(cn_src, cn_dest, (int)sa, (int)da, ps_value, rs_value, 
		 START_OF_FRAME, mode, 0, FRAME);
  cn_send_a_byte(cn_src, cn_dest, (int)sa, (int)da, ps_value, rs_value, 
		 ac, mode, 1, FRAME);
  cn_send_a_byte(cn_src, cn_dest, (int)sa, (int)da, ps_value, rs_value, 
		 fc, mode, 2, FRAME);
  cn_send_a_byte(cn_src, cn_dest, (int)sa, (int)da, ps_value, rs_value, 
		 da, mode, 3, FRAME);
  cn_send_a_byte(cn_src, cn_dest, (int)sa, (int)da, ps_value, rs_value, 
		 sa, mode, 4, FRAME);
  cn_send_a_byte(cn_src, cn_dest, (int)sa, (int)da, ps_value, rs_value,  
		 data, mode, 5, FRAME);
  cn_send_a_byte(cn_src, cn_dest, (int)sa, (int)da, ps_value, rs_value, 
		 END_OF_FRAME, mode, 6, FRAME);
  cn_send_a_byte(cn_src, cn_dest, (int)sa, (int)da, ps_value, rs_value, 
		 fs, mode, 7, FRAME);

  return ((caddr_t)cn_src);
}



/********************************************************************/
static caddr_t
cn_reset(cn)
   register NODEConnectiont *cn;
{

   return((caddr_t)cn);
}

/****************************************/
static caddr_t
cn_create(name)
   register char *name;
{

   NODEConnectiont *newcn;

   /* Memory for the component structure. */
   newcn = (NODEConnectiont *)sim_malloc(sizeof(NODEConnectiont));

   /* First things first-- copy name into the new structure. */
   strncpy(newcn->cn_name, name, 40);

   /* have to create a neighbor list */
   newcn->cn_neighbors= l_create();
   newcn->cn_params = q_create();
   newcn->cn_type = TOKEN_NODE_TYPE;
   newcn->cn_class = TOKEN_NODE_CLASS;
   newcn->cn_action = (PFP)token_node_action;
   newcn->cn_menu_up = FALSE;
   newcn->cn_flash = 0;

   /* Initialize the parameters */
   newcn->cn_Name = param_init ((Component*)newcn, "Name",
             (PFD)NULL, (PFP)make_name_text, (PFP)make_short_name_text,
             param_input_name,
             0, DisplayMask | InputMask | CanHaveLogMask, 0.0);

   newcn->cn_mode = param_init ((Component *)newcn,
             "Node Mode",
             (PFD)NULL, (PFP)make_double_text, (PFP)make_short_double_text,
              param_input_double,
             0, DisplayMask | InputMask , 0.0);
   newcn->cn_mode->u.i = 1;

   newcn->cn_is_monitor = param_init ((Component *)newcn,
             "Is_Monitor:",
             (PFD)NULL, (PFP)make_double_text, (PFP)make_short_double_text,
              param_input_double,
             0, DisplayMask | InputMask , 0.0);
   newcn->cn_is_monitor->u.i = 0;
 
   newcn->cn_having_error = param_init ((Component *)newcn,
             "Monitor having error",
             (PFD)NULL, (PFP)make_double_text, (PFP)make_short_double_text,
              param_input_double,
             0, DisplayMask | InputMask , 0.0);
   newcn->cn_having_error->u.i = 0;

   newcn->cn_delay = param_init ((Component *)newcn,
             "Delay(usecs)",
             (PFD)NULL, (PFP)make_double_text, (PFP)make_short_double_text,
              param_input_double,
             0, DisplayMask | InputMask , 0.0);
   newcn->cn_delay->u.i = 0;

   cise_register_node(newcn);

   return((caddr_t)newcn);
}


/*********************************************************/
static caddr_t
cn_neighbor(cn,c)
   register NODEConnectiont *cn;
   register Component *c;
{
   register Neighbor *n;

   if (!(n=add_neighbor((Component*)cn,c,2,1,LINK_CLASS)))
        return ((caddr_t)NULL);

   return ((caddr_t)n);
}

/*****************************************/
static caddr_t
cn_uneighbor(cn,c)
   register NODEConnectiont *cn;
   register Component *c;
{
   return ((caddr_t)remove_neighbor((Component *)cn,c));
}


/******************************************/
static caddr_t
cn_hops(cn)
   register NODEConnectiont *cn;
{
   register list *nlist;
   register Neighbor *n;

   nlist=l_create();

   for (n= (Neighbor *) cn->cn_neighbors->l_head;n;n=n->n_next)
        if (n->n_c->co_class == LINK_CLASS)
            l_addt(nlist,(caddr_t)n->n_c);

   return ((caddr_t)nlist);
}


/*******************************************/
static caddr_t
cn_route(cn,route_list)
   register NODEConnectiont *cn;
   list *route_list;
{

#ifdef NOTDEF
   Param *p;
   Component *this_component;
   Component *other_component;
   Neighbor *n1;
   Neighbor *n2;
    
   p=param_init((Component *)cn,"Other End Connection :               ",
                 int_calc,make_int_text,make_short_int_text,
                 (PFI)NULL,0,DisplayMask | CanHaveLogMask,0.0);

   if ((((Component *)route_list->l_head->le_data)->co_name[3] !=
          cn->cn_name[3]) || 
       (((Component *)route_list->l_head->le_data)->co_name[2] !=
          cn->cn_name[2]) || 
       (((Component *)route_list->l_head->le_data)->co_name[1] !=
          cn->cn_name[1]) || 
       (((Component *)route_list->l_head->le_data)->co_name[0] !=
          cn->cn_name[0]) ) 
       strcat(p->p_name,((Component *)route_list->l_head->le_data)->co_name);

   else strcat(p->p_name,((Component *)route_list->l_tail->le_data)->co_name);
 
   cn->cn_vpi=vpi;

   this_component = (Component *)route_list->l_head->le_data;
   other_component = (Component *)route_list->l_tail->le_data ;      

   n1= (Neighbor *)this_component->co_neighbors->l_head;
   n2= (Neighbor *)other_component->co_neighbors->l_head;

   if (n1->n_c->co_name[3] == n2->n_c->co_name[3])
       cn->cn_simple_connection=1;
         
   return ((caddr_t)cn); /* return something non-null */
#endif
}





int nodename_to_number(nodename)
  char *nodename;
{
  return((int)(nodename[4] - '0'));
}

void number_to_nodename(i, name)
  int i;
  char *name; 
{
  switch (i)
    {
    case 1:
      strcpy(name, "node1");
      break;
    
    case 2:
      strcpy(name, "node2");
      break;

    case 3:
      strcpy(name, "node3");
      break;

    case 4:
      strcpy(name, "node4");
      break;

    default:
      printf("number_to_nodename: should not be here\n");
    }
}

/* Used in display the color sector in the interface
   Normally, a node or link name corresponds to one or 
   several sectors in the ring */
int name_to_position(name, order)
     char *name;
     int order;
{
 

  if (!strcmp(name, "node1"))
    return 3;
  else if(!strcmp(name, "link1"))
    return (4+order);
  if (!strcmp(name, "node2"))
    return 7;
  else if(!strcmp(name, "link2"))
    return (8+order);
  if (!strcmp(name, "node3"))
    return 11;
  else if(!strcmp(name, "link3"))
    return (12+order);
  if (!strcmp(name, "node4"))
    return 15;
  else if(!strcmp(name, "link4"))
    return (0+order);
}

void show_message(char *format, ...)
{
    va_list args;
    char str[80];

    va_start( args, format );

    vsprintf( str, format, args );
    delete_mesg();
    show_mesg(str);
    dprintf(1, "%s", str);

    va_end( args );
}


Event *schedule_an_event(Evtype type, Component *src, Component *dest, 
	long time)
{
	return(ev_enqueue(type, src, dest, (tick_t) time, src->co_action, 
		NULL, (caddr_t)NULL));
}

