#include <stdio.h>
#include "stk.h"

#ifndef min
#define min(a,b)  ((a)<(b)?(a):(b))
#define max(a,b)  ((a)<(b)?(b):(a))
#endif

Stk Sx, Sr;

/* set the priorities of the token or frame according the proper priority
   value stored in the PPP and RRR fields 
   See the detailed algorithm on page 353 on Stalling's book
   Networking Standards -- A guide to OSI, ISDN, LAN, and MAN standards
   By William Stalling, Published by Addison Wesley'1993
   ISBN:0-201-56357-6 */
 
void Deal_With_Priority(Is_Token, pPf, pPs, pPr, pRs, pRr)
   int Is_Token, *pPf, *pPs, *pPr, *pRs, *pRr;
{

  int pdata;
  int top_of_sx, top_of_sr;
  int case_no;

  stkTop(&Sx, &top_of_sx);
  stkTop(&Sr, &top_of_sr);

  /* first case */
  if(Is_Token && *pPs <= *pPf) {
     *pRs = 0;
//     SendFrame();
     case_no = 1;
  }

  /* second case */
  else if(!Is_Token && *pPr >= max(*pPr, *pPf)) {
     *pPs = *pPf;
     *pRs = max(*pRr, *pPf);
     case_no = 2;
  }

  /* third case */
  else if(!Is_Token && *pPr < max(*pRr, *pPf) && *pPr > top_of_sx) {
     *pPs = max(*pRr, *pPf);
     *pRs = 0;
     stkPush(&Sr, *pPr);
     stkPush(&Sx, *pPs);
     case_no = 3;
  }

  /* fourth case */
  else if(!Is_Token && *pPr < max(*pRr, *pPf) && *pPr == top_of_sx) {
     *pPs = max(*pPr, *pPf);
     *pRs = 0;
     stkPop(&Sx, &pdata);
     stkPush(&Sx, *pPs);
     case_no = 4;
  }

  /* fifth case */
  else if((!Is_Token || (Is_Token && *pPf < top_of_sx)) &&
     (*pPs == top_of_sx) && (*pRr > top_of_sr)) {
     *pPs = *pRr;
     *pRs = 0;
     stkPop(&Sx, &pdata);
     stkPush(&Sx, *pPs);
     case_no = 5;
  }

  /* Sixth case */
  else if((!Is_Token || (Is_Token && *pPf < top_of_sx)) &&
     (*pPs == top_of_sx) && (*pRr <= top_of_sr)) {
     *pPs = *pRr;
     *pRs = 0;
     stkPop(&Sr, &pdata);
     stkPop(&Sx, &pdata);
     case_no = 6;
  }

  printf("Pf=%d, Ps=%d, Pr=%d, Rs=%d, Rr=%d, case=%d\n",  
          *pPf, *pPs, *pPr, *pRs, *pRr, case_no);
  
  return;
}

main()
{
  int pf=1, ps=3, pr=0, rs=7, rr=5;
  int Is_Token = 0;

  stkInit(&Sx);
  stkInit(&Sr);
  stkPush(&Sx, 4);
  stkPush(&Sr, 3);
  stkPush(&Sx, 5);
  stkPush(&Sr, 6);
  stkPush(&Sx, 2);
  stkPush(&Sr, 7);
  stkPush(&Sx, 0);
  stkPush(&Sr, 3);
  stkPush(&Sx, 3);
  stkPush(&Sr, 2);

  Deal_With_Priority(Is_Token, &pf, &ps, &pr, &rs, &rr);
  Deal_With_Priority(Is_Token, &pf, &ps, &pr, &rs, &rr);
  Deal_With_Priority(Is_Token, &pf, &ps, &pr, &rs, &rr);
  Deal_With_Priority(Is_Token, &pf, &ps, &pr, &rs, &rr);
  Deal_With_Priority(Is_Token, &pf, &ps, &pr, &rs, &rr);
  Deal_With_Priority(Is_Token, &pf, &ps, &pr, &rs, &rr);
  Deal_With_Priority(Is_Token, &pf, &ps, &pr, &rs, &rr);
  Deal_With_Priority(Is_Token, &pf, &ps, &pr, &rs, &rr);

  return 0;
}


