/* +++Date last modified: 02-Nov-1995 */

/*
**  STK.C - Stack manager
**
**  by Dustin Puryear (dpuryear@intersurf.com), placed into Public Domain.
**  stkTop by CAI on 7/27/97
*/

#include <stdio.h>       /* General include file */
#include <stdlib.h>      /* malloc() and free()  */
#include "stk.h"         /* Function prototypes  */

void stkInit(pstack)
   Stk *pstack;
{
      pstack->ptop = NULL;
      pstack->vcount = 0;
}

int stkPush(pstack, pdata)
   Stk *pstack;
   int pdata;
{
      StkNode *padd;   /* Node to add to top of stack */

      /*
      ** Create the node to append to the top of the stack. Also,
      ** check for a memory allocation error.
      */

      padd = (StkNode *) malloc(sizeof(StkNode));
      if ( padd == NULL )
      {
            return (0);
      }

      /*
      ** Now, add the data pointer to the node.
      */

      padd->pdata = pdata;

      /*
      ** Manipulate the stack so that the new node is on top, and
      ** increment count.
      */

      if ( pstack->ptop == NULL )
      {
            padd->pprev = NULL;   /* Anchor it!  */
            pstack->ptop = padd;
      }
      else
      {
            padd->pprev = pstack->ptop;
            pstack->ptop = padd;
      }
      pstack->vcount++;

      return (1);
}

int stkPop(pstack, ppdata)
   Stk *pstack;
   int *ppdata;
{
      StkNode *pdel;   /* Pointer to node to remove (top) */

      /*
      ** Check to see if stack is empty.
      */

      if ( pstack == NULL || pstack->vcount == 0)
      {
            ppdata = NULL;
            return (0);
      }

      /*
      ** Pop data pointer value into ppdata (pointer to user's data)
      */

      *ppdata = pstack->ptop->pdata;

      /*
      ** Pop the top of the stack off and make the next in line top.
      ** Also, decrement counter.
      */

      pdel = pstack->ptop;
      pstack->ptop = pstack->ptop->pprev;
      free(pdel);
      pstack->vcount--;

      return (1);
}


int stkTop(pstack, ppdata)
   Stk *pstack;
   int *ppdata;
{

      /*
      ** Check to see if stack is empty.
      */

      if ( pstack == NULL || pstack->vcount == 0)
      {
            ppdata = NULL;
            return (0);
      }

      /*
      ** Pop data pointer value into ppdata (pointer to user's data)
      */

      *ppdata = pstack->ptop->pdata;

      return(1);
}


unsigned stkCount(pstack)
   Stk *pstack;
{
      return (pstack->vcount);
}
