/* Main routine of the CISE simulator.                 */
/* modified from main.c,v 1.0 1993/10/21 19:11:57 NIST */
/*
   $Log:	main.C,v $
 * Revision 4.2  96/09/25  17:10:51  tlin
 * foo
 * 
   Revision 4.1  1996/09/20 06:44:49  tlin
   dprintf()

   Revision 4.0  1996/08/23 20:02:27  tlin
   bump to mickey mouse version

   Revision 3.22  1996/06/30 20:25:04  tlin
   add FOR_DEMO

 * Revision 3.21  96/06/30  01:02:31  tlin
 * add back get_argv() and -x option
 * 
 */


#define MAIN                    /* This file contains main() */

#include "cisePort.h"
// extern "C" {
#include "sim.h"
#include "component.h"
#include "event.h"
#include "eventdefs.h"
#include "pdu.h"
#include "main.h"
#include "file.h"
#include "route.h"
#include "random.h"
// };


/* moved from simx.h -- tlin@cis */
extern list *route_table;


int sim_reset(list *comps)
{
  Component *c;
  
  ev_reset();			/* Clear out any pending events */

  for (c = (Component *) comps->l_head; c; c = c->co_next)
      (*c->co_action)(NULL, c, EV_RESET,NULL,NULL);

return 0;
 
}

/* Start to all components */
int sim_start(list *comps)
{
    Component *c;
    for (c = (Component *) comps->l_head; c; c = c->co_next)
	(*c->co_action)(NULL, c, EV_START,NULL,NULL);

	return 0;
}



int  start_tk(int argc, char **argv)
{
    char *new_argv[40];
    char script[256];
    int  i;

    M.libDir = getenv("CISE_LIBRARY_TOKEN_RING");   // needs to be redo in cise.C because the result 
    if (M.libDir == NULL) {              // from getenv() is not permanent
       // M.libDir = CISE_LIBRARY;         // we need this right now for the initial script
		M.libDir ="D:\\CISE\\DEVELOPING\\token-ring\\SIM.SRC";

    }
    strcpy(script, M.libDir);
    strcat(script, "/sim.tcl");
    
    if (argc>39) argc=39;
    new_argv[0]=argv[0];
    new_argv[1]=script;                  // Tk_Main() never returns so script[] is permanent
    for (i=1; i<argc; i++) 
	new_argv[i+1]=argv[i];

    if (_access(script, 04)!=0) {
	fprintf(stderr,"Cannot find the script file %s. \n  Did you set the "
		"CISE_LIBRARY environment variable correctly?\n", script);
	exit(2);
    }

    Tk_Main(argc+1, new_argv, Tcl_AppInit);

	return 0;
}


void init_cise()
{
    M.doX=1;
    M.seed=0;
    dbg_set_level(DBG_ERR);
    M.filename=NULL;
    M.comps=NULL;
    M.libDir=NULL;

    M.stoptime = (unsigned long)~0;
    M.delay    = 0;
    M.running  = 0;
    M.paused   = 0;
    M.single_step = 0;

    /* Create snap file name by appending the PID to "sim_snap."  */
    /* If the simulator runs longer than stoptime, it will snapshot */
    /* its current state into this file before exiting */
    sprintf(M.snapname, "sim_snap.%d", getpid());

    /* X related stuff */
    M.reverse_video = 0;
    M.interp        = NULL;          /* Tcl/Tk interpreter           */
    M.bg_name       = NULL;          /* name of background of canvas */

    M.debugLevel    = 0;
}

int init_simulator()
{
    /* Event manager */
    if (!ev_init())  {
	dbg_write(DBG_ERR, NULL, "Dying: ev_init failed");   
	exit(1);
    }
    if (!pdu_init())  {
	dbg_write(DBG_ERR, NULL, "Dying: pdu_init failed");  
	exit(1);
    }

    if (!rt_create())  {
	dbg_write(DBG_ERR, NULL, "Dying: rt_create failed");
	exit(1);
    }

    if (!M.seed)	/* If seed not specified, use the time */
	M.seed = (int)time((long *)NULL);
    fprintf(stderr, "Seed is: %d\n", M.seed);
    Cise_srandom(M.seed);

    if (!route_table)
	route_table = l_create();

    if (!M.comps)
	M.comps = l_create();

	return 0;
}


/* specialized to deal with -x only */
int get_argv(int *argc, char **argv)
{
    

    if (*argc<2)                 return 0;
    if (strcmp(argv[1],"-h")==0) usage(argv);
    if (strcmp(argv[1],"-x")!=0) return 0;
    if (*argc!=6)                usage(argv);
    M.doX=0;

    M.pr_name = argv[0];
   
	if ((M.pr_name=strrchr(argv[0],'/'))!=NULL) M.pr_name++;


    if (strcmp(argv[2],"-s")!=0)            usage(argv);
    M.seed = atoi(argv[3]);
    M.filename=argv[4];                     /* world file name */
    M.stoptime = (tick_t)atoi(argv[5]);     /* stop time       */
    /* This means that we can never go more than 2**32-1 usecs, but
       many other parts of the simulator have this problem already anyway. */
    M.stoptime = USECS_TO_TICKS(M.stoptime);

    if ( (M.comps=read_world(M.filename))==NULL ) {
        printf("Unable to load file %s\n", M.filename);
	exit(1);
    } else 
        printf("File %s loaded\n", M.filename);


}

int usage(char *argv[])
{
    printf("Usage: %s -x -s <seed> worldfile stoptime\n", argv[0]);
    printf("   or: %s [-s seed] [worldfile [stoptime]]\n", argv[0]);
    printf("   or: %s -h          -- show this message\n", argv[0]);
    exit(1);

	return 0;
}


void dprintf( int level, char *format, ...)
{
    va_list args;

    va_start( args, format );
    
    if (M.debugLevel>=level) {
        vfprintf( stdout, format, args );
    }
    va_end( args );
}

int APIENTRY WinMain(HINSTANCE hINstance, 
					 HINSTANCE hPrevInstance,
					 LPSTR     lpszCmdLine,
					 int       nCmdShowint)
{
    
    char *argv[10];
	int   argc=0;
	char buffer[20];

    /*
	 * Set up the argv, and argv
	 */
	GetModuleFileName(NULL, buffer, sizeof(buffer));
	argv[0]=buffer;
	argv[1]=lpszCmdLine;
	argc   =nCmdShowint+1;

	init_cise();    
    init_simulator();  
    get_argv(&argc, argv);

    start_tk(argc, argv);


    printf("Should never come here\n");
    
    /* will this loop stop when no events left? I doubt. -- tlin@cis */
    /* from netsim anyway. change it later.                          */
    for (;;)  {
        printf("now=%ld\n", ev_now());
	    if (ev_now() > M.stoptime) 
	       exit(0);
	 
	    /* Make all the events that happen at the same time fire.  */
	    if (M.single_step)  
	       ev_fire_first();  
	    else  
	       ev_fire_all_first();
    }  
	
	return 0;
} 
