

/*
   $Log:	route_activity.C,v $
Revision 3.0  97/01/31  00:26:46  goyal
Final Revision

Revision 2.0  97/01/31  00:25:34  goyal
Final Revision

Revision 1.1  97/01/18  17:14:08  goyal
Initial revision


   Revision 1.1  1996/06/02 05:14:23  tlin
   Initial revision
   */

/* was in lines.c -- developed by tlin@cis */

#include "cisePort.h"
extern "C" {
#include "q.h"
#include "list.h"
#include "hash.h"
#include "sim.h"
#include "eventdefs.h"
#include "component.h"
#include "main.h"
#include "sim_tk.h"
#include "route_activity.h"
};

#define SEEXNOW XFlush(M.display) 

/* ------------------------------------------------------------------

   route_activity() by tlin@cis 3/96
   draw blocks for packets instead of flashing the route only

   ------------------------------------------------------------------ */


#define KEYSIZE  sizeof(XComponent*)*2
#define MAXDASH  20  
#define MAXROUTE 14   /* see also xinit.c */


#define MAX_ACTIVE_ROUTES 97   /* for hash table; prime # preferred */
#define MAX_PACKET_IN_LINK 50  /* MAX # of packets in one direction of link */


typedef struct { 
    int   from_x, from_y, to_x, to_y, last_x, last_y;
    int   age, error;
    ulong delay;
    ulong start_time;
    int   color;
} packet_activity_t;

typedef struct { 
    packet_activity_t packet[MAX_PACKET_IN_LINK];
    int    x1, y1, x2, y2;        /* from (x1,y1) to (x2,y2)       */
    int    h, t, qn;              /* head, tail, #element of queue */
} route_activity_t;

static Htable HTRouteActivity=NULL;  /* hash the address of RouteActivity */


 



int init_route_activity()
{
    delete_route_activity();
    HTRouteActivity=
	ht_create(MAX_ACTIVE_ROUTES, sizeof(XComponent*)*2, 
		  sizeof(route_activity_t));      
    
	return 0;
}

int delete_route_activity()
{
    if (HTRouteActivity!=NULL) 
	ht_obliterate(HTRouteActivity);
    HTRouteActivity=NULL;

	return 0;
}

 

 

int BlockW=16, BlockH=16;

#define set_fg(gc,fg) \
    XSetForeground(M.display,gc,fg)
#define set_bg(gc,bg) \
    XSetBackground(M.display,gc,bg)

#define draw_block(gc,x,y) \
     XFillRectangle(M.display, M.canvas, gc, x, y, BlockW, BlockH)

void make_new_route(XComponent *from, XComponent *to, ulong delay, 
		    route_activity_t *rt);

int route_activity(Component *src, Component *dest, tick_t delay, 
		       int color, int error)
{
    XComponent *from=(XComponent *) src->co_picture;
    XComponent *to  =(XComponent *)dest->co_picture;
    char       key[KEYSIZE];
    route_activity_t  *rt; 
    packet_activity_t *pkt;

    memcpy(key,                     &from, sizeof(XComponent*));
    memcpy(key+sizeof(XComponent*), &to,   sizeof(XComponent*));
    rt=(route_activity_t*)ht_find(HTRouteActivity, key);
    if (rt==NULL) {
	route_activity_t new_rt;
	make_new_route(from, to, delay, &new_rt);
	ht_insert(HTRouteActivity, key, (char*)&new_rt);
	rt=(route_activity_t*)ht_find(HTRouteActivity, key);
    }
  
    if (rt->qn>=MAX_PACKET_IN_LINK) {
	 dprintf(3,"Link can hold no more than %d packets. Did not update.\n", 
	   MAX_PACKET_IN_LINK);
	return -1;
    }
    pkt=&(rt->packet[rt->h++]);
    rt->h%=MAX_PACKET_IN_LINK;
    rt->qn++; 
  
    pkt->from_x=rt->x1;
    pkt->from_y=rt->y1;
    pkt->to_x  =rt->x2;
    pkt->to_y  =rt->y2;
    pkt->delay =delay;
    pkt->color =color;
    pkt->start_time=ev_now();

    
    draw_block(M.gcxor[color], pkt->from_x, pkt->from_y);
    pkt->last_x=pkt->from_x;
    pkt->last_y=pkt->from_y;
    pkt->age=0;
    pkt->error=-1;
    if (error) pkt->error=rand()%4+1;
    ev_enqueue(EV_UPDATE_ROUTE_ACTIVITY, src, dest, ev_now()+delay/5,
	       src->co_action, NULL, NULL);
    /*flash_component(src);*/
    return 0;
}

int update_routes(Component *src, Component *dest)
{
    XComponent        *from=(XComponent *) src->co_picture;
    XComponent        *to  =(XComponent *)dest->co_picture;
    char              key[KEYSIZE];
    route_activity_t  *rt;
    packet_activity_t *pkt;
    int               j, h, qn;

    memcpy(key,                     &from, sizeof(XComponent*));
    memcpy(key+sizeof(XComponent*), &to,   sizeof(XComponent*));
    rt=(route_activity_t*)ht_find(HTRouteActivity, key);
    if (rt==NULL) {
	printf("Internal error: update non-existing route.\n");
	return -1;
    }
  
    qn=rt->qn;
    for (j=0, h=rt->t; j<qn; j++, h=(h+1)%MAX_PACKET_IN_LINK) {
	int   x, y, fx, fy, tx, ty, color;
	float dt;
	pkt=&(rt->packet[h]);

	if (pkt->error==-1 || pkt->age < pkt->error) {
	    color=pkt->color;
	    draw_block(M.gcxor[color],  pkt->last_x, pkt->last_y);
	} else if (pkt->age==pkt->error) {
	    
	}

	if (pkt->start_time+pkt->delay < ev_now()) {
	    /* destroy this packet */
	    rt->qn--;
	    rt->t=(rt->t+1)%MAX_PACKET_IN_LINK;
	    continue;
	}

	fx=pkt->from_x; 
	fy=pkt->from_y; 
	tx=pkt->to_x; 
	ty=pkt->to_y; 
	dt=(float)(ev_now()-pkt->start_time);
	x=(int)((tx-fx)*dt/pkt->delay)+fx;
	y=(int)((ty-fy)*dt/pkt->delay)+fy;
    
	pkt->age++;
	if (pkt->error==-1 || pkt->age<pkt->error) {
	    draw_block(M.gcxor[color], x, y);
	} else if (pkt->age==pkt->error) {
 
	}
	pkt->last_x=x;
	pkt->last_y=y;
    }
  
    if (rt->qn!=0) {   /* some packets still needs to be taken care of */
	tick_t next_ev_time=ev_now()+pkt->delay/5;
	if (next_ev_time<ev_now()) /* make the "stub" of block disappear asap */
	    next_ev_time=ev_now()+1;
	ev_enqueue(EV_UPDATE_ROUTE_ACTIVITY, src, dest, next_ev_time,
		   src->co_action, NULL, NULL);
    }

	return 0;
}

static void conn_point(int x, int y, int w, int h, int which, int *px, int *py)
{
    *px=x; *py=y;
    switch (which) {
    case 0:
	*px+=w/2; break;
    case 1:
	*px+=w; *py+=h/2; break;
    case 2:
	*px+=w/2; *py+=h; break;
    case 3:
	*py+=h/2; break;
    case 4:
	*px+=w/4; break;
    case 5:
	*px+=w*3/4; break;
    case 6:
	*px+=w; *py+=w/4; break;
    case 7:
	*px+=w; *py+=w*3/4; break;
    case 8:
	*px+=w*3/4; *py+=w; break;
    case 9:
	*px+=w/4; *py+=w; break;
    case 10:
	*py+=w*3/4; break;
    case 11:
	*py+=w/4; break;
    }
    *px-=BlockW; *py-=BlockH;        /* half of the visual block size */

	
}

void make_new_route(XComponent *from, XComponent *to, ulong delay, 
		    route_activity_t *rt)
{
    int dx, dy, p_from, p_to;
     

    rt->h=rt->t=rt->qn=0;
  
    dx=to->x-from->x;
    dy=to->y-from->y;

    /* a (seemingly) complex algo to compute the points of attachments */
    if (dx>0 && dy<0) {
	if (dx<5) {
	    p_from=5; p_to=8;
	} else if (dy>-5) {
	    p_from=7; p_to=10;
	} else 
	    p_from=1; p_to=2;
    } else if (dx>0 && dy>0) {
	if (dx<5) {
	    p_from=9; p_to=4;
	} else if (dy<5) {
	    p_from=7; p_to=10;
	} else 
	    p_from=2; p_to=3;
    } else if (dx<0 && dy>0) {
	if (dx>-5) {
	    p_from=9; p_to=4;
	} else if (dy<5) {
	    p_from=11; p_to=6;
	} else 
	    p_from=3; p_to=0;
    } else if (dx<0 && dy<0) {
	if (dx>-5) {
	    p_from=5; p_to=8;
	} else if (dy>-5) {
	    p_from=11; p_to=6;
	} else 
	    p_from=0; p_to=1;
    } else if (dx==0) {
	if (dy>0) {
	    p_from=9; p_to=4;
	} else {
	    p_from=5; p_to=8;
	}
    } else if (dy==0) {
	if (dx>0) {
	    p_from=7; p_to=10;
	} else {
	    p_from=11; p_to=6;
	}
    }

    conn_point(from->x,from->y,from->w,from->h,p_from,&rt->x1,&rt->y1);
    conn_point(  to->x,  to->y,  to->w,  to->h,p_to,  &rt->x2,&rt->y2);
}


//
// this refresh_route_activity() is ugly looking. Can someone
// make it better? -- tlin@cis
//
#include <tk.h>
struct {
    int               x1, y1, x2, y2;
} Exp;  /* for exposure event */


void refresh_route_activity1(ClientData clientData);
void refresh_route_activity2();

void refresh_route_activity(ClientData clientData, XEvent *eventPtr)
{
    XExposeEvent      *xevent=&eventPtr->xexpose;
    
    Exp.x1=xevent->x; Exp.x2=Exp.x1 + xevent->width;
    Exp.y1=xevent->y; Exp.y2=Exp.y1 + xevent->height;

    if (xevent->count!=0) return;
    Tk_DoWhenIdle(refresh_route_activity1, 0);
}

void refresh_route_activity1(ClientData clientData)
{
    static c=0;
    
    if (c%3)    // experimented value
	Tk_DoWhenIdle(refresh_route_activity1, 0);
    else
	refresh_route_activity2();
}

void refresh_route_activity2()
{
    char              *ch;
    route_activity_t  *rt; 
    packet_activity_t *pkt;

    if (HTRouteActivity==NULL) return;
    ht_iter_setup(HTRouteActivity);
    while ((ch=ht_iter(HTRouteActivity))!=NULL) {
	int qn, j, h;
	rt=(route_activity_t*)(ch+KEYSIZE); 
	qn=rt->qn;
	for (j=0, h=rt->t; j<qn; j++, h=(h+1)%MAX_PACKET_IN_LINK) {
	    pkt=&(rt->packet[h]);
	    if (pkt->last_x < Exp.x1 || pkt->last_x > Exp.x2 ||
		pkt->last_y < Exp.y1 || pkt->last_y > Exp.y2) 
		continue;
	    if (pkt->error==-1 || pkt->age<pkt->error) {
		draw_block(M.gcxor[pkt->color], pkt->last_x, pkt->last_y);
	    } else if (pkt->age==pkt->error) {
	 
	    }
	}
    }
}

int flash_component(Component *c)
{
  

/*  not ready yet -- tlin@cis 6/2/96
      XFillRectangle
	(M.display, xc->comp_window,
	 the_environment.route_gc[0][1],
	 0, 0, xc->width, xc->height);
	 */

	return 0;
}
