

# $Log: sim.tcl,v $
# Revision 1.2  1997/05/11 16:02:50  shihwei
# add hello, ptsp to icon_color()
#
# Revision 1.1  1997/05/10 23:34:51  shihwei
# Initial revision
#
# Revision 4.5  1996/09/25 19:50:15  tlin
# about{}
#
# Revision 4.4  1996/09/25 17:16:56  tlin
# debugLevel: 0-3
#
# Revision 4.3  1996/09/20 07:02:14  tlin
# foo
#
# Revision 4.2  1996/09/20 06:58:43  tlin
# add $MdebugLevel
#
# Revision 4.1  1996/09/19 06:19:17  tlin
# split space-time.tcl for space-time diagram
#
# Revision 4.0  1996/08/23 20:02:27  tlin
# bump to mickey mouse version
#
# Revision 3.39  1996/07/31 17:03:37  tlin
# foo
#
#


#
#
# driving script for cise project -- tlin@cis, 5/26/96
#
#

# include some useful subroutines
source $cise_library/subr.tcl
source $cise_library/space-time.tcl
source $cise_library/token-ring.tcl

# ------------------------------------------------------
#
#      general outlook of the window
#
# ------------------------------------------------------

wm geometry . +50+30
wm title . "Cise Project"

set width  800
set height 650


source $cise_library/layout.tcl


# ------------------------------------------------------
#
#      menu commands 
#
# ------------------------------------------------------


proc nop {name} {
    .msg configure -text "Command $name: this function is not ready yet"
}

proc menu_load {filename} {
    global Mfilename
    if {$filename == ""} {
	set Mfilename [input .d "Wait for input" "Please input filename:" $Mfilename]
    } else {
	set Mfilename $filename
    }
    .filename configure -text "Filename: $Mfilename"
    .msg      configure -text "[cise_load]"
}

proc menu_run {} {
    bind . <space> {menu_step1}
    .msg configure -text "[cise_run]; <space> for single step"
}

proc menu_step {} {
    bind . <space> {menu_step1}
    menu_step1
}

proc menu_step1 {} {
    .msg configure -text "[cise_step]; use \"Resume\" to resume; <space> for more";
}

proc menu_pause {} {
    .msg configure -text [cise_pause]
}

proc menu_resume {} {
    bind . <space> {}
    .msg configure -text [cise_resume]
}

proc menu_delay {} {
    set t [cise_delay]
    set t [input .d "Wait for input" "Please input amount of delay (msec)" $t]
    change_delayTime $t
}

proc menu_stopTime {} {
    set t [expr [cise_stopTime]/1000]
    set t [input .d "Wait for input" "Please input Stop Time (usec)" $t]
    change_stopTime $t
}

proc about {} {
    dialog .about {About CISE} \
	    {CISE project was developed at CIS, 
The Ohio State Univ, 1995-1998.

Principle Investigator:  Steve Lai
		       Raj Jain
Developers:    Shih-wei Li
                        Xingrong Cai
               } \
	    info -1 OK
}

proc how_to_use {} {
    global cise_library
    xnotify .d "Help: How to use" "$cise_library/how-to-use.txt" -1 "OK"
}


# ------------------------------------------------------
#
#      draw graph for objects
#
# ------------------------------------------------------

set icon_color(link)   green
set icon_color(bomb)   red
set icon_color(node)   tan
set icon_color(prg)    pink
set icon_color(ct)     gold
set icon_color(amp)    yellowgreen
set icon_color(frame)  darkorchid
set icon_color(empty)  $Mbg_name
set icon_color(token)  red
set icon_color(monitor) gold
set monitorname        node1     

proc make_notefoot {} {
    global icon_color 
    
    .c create rectangle 700 200 800 220 -fill $icon_color(token)
    .c create text      830 210  -text "Token"  -fill red -anchor w
    .c create rectangle 700 240 800 260 -fill $icon_color(frame)
    .c create text      830 250  -text "Frame"  -fill red -anchor w
    .c create rectangle 700 280 800 300 -fill $icon_color(prg)
    .c create text      830 290  -text "Purge" -fill red  -anchor w
    .c create rectangle 700 320 800 340 -fill $icon_color(ct)
    .c create text      830 330  -text "Claim Token"  -fill red  -anchor w
    .c create rectangle 700 360 800 380 -fill $icon_color(amp)
    .c create text      830 370  -text "Active Monitor Present" -fill red  -anchor w
    
}

proc make_icon {name type x y} {
    global icon_color icon_name 
        
     .c create rectangle [expr $x-17] [expr $y-17] [expr $x+17] [expr $y+17] \
	    -fill $icon_color($type) -tag icon$name 
    
    set icon_id [ .c create text $x $y -text $name -tag icontext$name]
    set icon_name($icon_id) $name
    
   
}

proc show_who_is_monitor {name} {

    global monitorname

    .c itemconfigure icon$monitorname -fill $icon_color(node)
    
    set monitorname $name
    .c itemconfigure icon$name -fill $icon_color(monitor)
    update
}


proc show_neighbor_relation {x1 y1 x2 y2 name1 name2} {
    .c create line $x1 $y1 $x2 $y2 -fill white -tag $name1$name2
    .c lower $name1$name2
}



# ------------------------------------------------------
#
#      status line
#
# ------------------------------------------------------

proc update_time {} {
    .time configure -text "[cise_time] usec"
}

proc change_stopTime {t} {
    cise_stopTime [expr $t*1000]
    set a [cise_stopTime]
    .stopTime configure -text "Stop Time: [expr $a/1000] usec"
}
change_stopTime 100000

proc change_delayTime {t} {
    cise_delay $t
    .delayTime configure -text "Delay: [cise_delay] msec"
}
change_delayTime 50

proc menu_debugLevel {inc} {
    global MdebugLevel
    incr MdebugLevel $inc
    if {$MdebugLevel<0} {set MdebugLevel 0}
    if {$MdebugLevel>3} {set MdebugLevel 3}
    .debugLevel configure -text "DebugLevel: $MdebugLevel"
}


# ------------------------------------------------------
#
#      main program
#
# ------------------------------------------------------


# show all windows so menu_load{} has a canvas to draw on
update

# some variables (eg. canvas .c) are not created yet when 
# Cise_Init() is invoked. cise_init2 takes care of these.
 cise_init2

 set Mfilename ""

if {$argc>=1} {
    menu_load [lindex $argv 0]
}
















