
#
# Token Ring's GUI
#
# 
# Developed by Shih-wei Li   sli@cis.ohio-state.edu
#


set stack(width)  18
set stack(height) 48
set stack(unit)    12  
set stack(dis1)    25
set stack(dis2)    50
set stack(color)  yellow

# destination stack
set dest_stack_db      {} 
# priority stack
set pri_stack_db       {}

# draw dest stack
proc draw_dest_stack {name x y} {
    global stack
    
    set base_x [expr $x+$stack(dis1)]
    set base_y $y 
    
    .c create line $base_x                      [expr $base_y-$stack(height)] \
	           $base_x                      $base_y                       \
		   [expr $base_x+$stack(width)] $base_y                       \
		   [expr $base_x+$stack(width)] [expr $base_y-$stack(height)] \
		    -tag $name.dest_stack
    .c create text $base_x [expr $base_y + 4] -anchor w -text "dest." -tag $name.dest_stack
}

# draw dest stack
proc draw_pri_stack {name x y} {
    global stack
    
    set base_x [expr $x+$stack(dis2)]
    set base_y $y 
    
    .c create line $base_x                      [expr $base_y-$stack(height)] \
	           $base_x                      $base_y                       \
		   [expr $base_x+$stack(width)] $base_y                       \
		   [expr $base_x+$stack(width)] [expr $base_y-$stack(height)] \
		    -tag $name.pri_stack

    
     .c create text $base_x [expr $base_y + 3] -text "pri." -tag $name.pri_stack
}

# draw elements in dest stack
# (x,y) = where node is
# 
proc display_elements_in_stack {name x y num} {
    global stack stack_db
    
    set base_x [expr $x+$stack(dis1)]
    set base_y $y

    
    .c delete $name.stack.num

    for {set i 1} {$i <= $num} {incr i 1} {
	.c create rectangle $base_x           [expr $base_y-$i*$stack(unit)] \
		[expr $base_x+$stack(width)]  [expr $base_y-($i-1)*$stack(unit)] \
		-fill $stack(color) -outline red -tag $name.stack.num
    }
    
}


#
# Tools for manipulate destination stack
#
proc push_dest_element {name x y dest} {
    global stack dest_stack_db
    
    set base_x [expr $x+$stack(dis1)]
    set base_y $y
    set db {}

    set dest_stack_db " $dest_stack_db [list $name$dest]"

    foreach el $dest_stack_db {
	if [string match $name* $el] {
	    lappend db $el
	}
    }


    
     .c delete $name.dest_stack.num

    set num [llength $db]

    for {set i 1} {$i <= $num} {incr i 1} {
	.c create rectangle $base_x           [expr $base_y-$i*$stack(unit)] \
		[expr $base_x+$stack(width)]  [expr $base_y-($i-1)*$stack(unit)] \
		-fill $stack(color) -outline red -tag $name.dest_stack.num
	.c create text [expr $base_x+$stack(width)/2] [expr $base_y-($i-0.5)*$stack(unit)] \
		-text [string range [lindex $db [expr $i -1]] [string length $name] end] \
		-fill black -tag  $name.dest_stack.num
    }
}

proc pop_dest_element {name x y} {
    global stack dest_stack_db
    
    set base_x [expr $x+$stack(dis1)]
    set base_y $y
    set db {}
    
    foreach el $dest_stack_db {
	if [string match $name* $el] {
	    lappend db $el
	}
    }
    
    set pos [lsearch $dest_stack_db [lindex $db [expr [llength $db] - 1]]]
    set dest_stack_db [lreplace $dest_stack_db $pos $pos]
    set pos [expr [llength $db] - 1]
    set db  [lreplace $db $pos $pos] 

     .c delete $name.dest_stack.num

    set num [llength $db]

    for {set i 1} {$i <= $num} {incr i 1} {
	.c create rectangle $base_x           [expr $base_y-$i*$stack(unit)] \
		[expr $base_x+$stack(width)]  [expr $base_y-($i-1)*$stack(unit)] \
		-fill $stack(color) -outline red -tag $name.dest_stack.num
	.c create text [expr $base_x+$stack(width)/2] [expr $base_y-($i-0.5)*$stack(unit)] \
		-text [string range [lindex $db [expr $i -1]] [string length $name] end] \
		-fill black -tag  $name.dest_stack.num
    }
}

#
# Tools for manipulate priority stack
#
proc push_pri_element {name x y pri} {
    global stack pri_stack_db
    
    set base_x [expr $x+$stack(dis2)]
    set base_y $y
    set db {}

    set pri_stack_db " $pri_stack_db [list $name$pri]"

    foreach el $pri_stack_db {
	if [string match $name* $el] {
	    lappend db $el
	}
    }


    
     .c delete $name.pri_stack.num

    set num [llength $db]

    for {set i 1} {$i <= $num} {incr i 1} {
	.c create rectangle $base_x           [expr $base_y-$i*$stack(unit)] \
		[expr $base_x+$stack(width)]  [expr $base_y-($i-1)*$stack(unit)] \
		-fill $stack(color) -outline red -tag $name.pri_stack.num
	.c create text [expr $base_x+$stack(width)/2] [expr $base_y-($i-0.5)*$stack(unit)] \
		-text [string range [lindex $db [expr $i -1]] [string length $name] end] \
		-fill black -tag  $name.pri_stack.num
    }
}

proc pop_pri_element {name x y} {
    global stack pri_stack_db
    
    set base_x [expr $x+$stack(dis2)]
    set base_y $y
    set db {}
    
    foreach el $pri_stack_db {
	if [string match $name* $el] {
	    lappend db $el
	}
    }
    
    set pos [lsearch $pri_stack_db [lindex $db [expr [llength $db] - 1]]]
    set pri_stack_db [lreplace $pri_stack_db $pos $pos]
    set pos [expr [llength $db] - 1]
    set db  [lreplace $db $pos $pos] 

     .c delete $name.pri_stack.num

    set num [llength $db]

    for {set i 1} {$i <= $num} {incr i 1} {
	.c create rectangle $base_x           [expr $base_y-$i*$stack(unit)] \
		[expr $base_x+$stack(width)]  [expr $base_y-($i-1)*$stack(unit)] \
		-fill $stack(color) -outline red -tag $name.pri_stack.num
	.c create text [expr $base_x+$stack(width)/2] [expr $base_y-($i-0.5)*$stack(unit)] \
		-text [string range [lindex $db [expr $i -1]] [string length $name] end] \
		-fill black -tag  $name.pri_stack.num
    }
}


# draw the display window for each node
proc display_window {name x y} {    
    .c create rectangle [expr $x-45] [expr $y-30] [expr $x-25] [expr $y-10] \
	    -fill yellow -outline red -tag $name.win

}

proc display_number_in_window {name x y num} {
    .c delete $name.win.num
    .c create text [expr $x-35] [expr $y-20] -text $num -anchor center -tag $name.win.num
}

proc delete_number_in_window {name} {
    .c delete $name.win.num
}



#
# Graphy Information for drawing Token_Ring
#
#      (x1,y1)
#      +-----------------------+
#      |                       |
#      |                       |
#      |       Token Ring      |
#      |                       |
#      |                       |
#      |                       |
#      +-----------------------+ (x2,y2)

# default values for the token ring
set token_ring(x1) 50
set token_ring(y1) 50
set token_ring(x2) 350
set token_ring(y2) 350
set token_ring(delta) 20
set token_ring(semi_dia) 100 

# draw the token ring
proc show_token_ring {cx cy semi_dia} {
    global token_ring icon_color
    
    set token_ring(semi_dia) $semi_dia
    set token_ring(x1) [expr $cx-$token_ring(semi_dia)]
    set token_ring(y1) [expr $cy-$token_ring(semi_dia)]
    set token_ring(x2) [expr $cx+$token_ring(semi_dia)]
    set token_ring(y2) [expr $cy+$token_ring(semi_dia)]

 
    for {set i 0} {$i < 16} {incr i 1} {
	.c create arc $token_ring(x1) $token_ring(y1) $token_ring(x2) $token_ring(y2) \
	    -start [expr $i*22.5+11.25] -extent 22.5 -fill $icon_color(empty) -outline black \
	    -style pieslice -tag token_ring.$i 
	.c lower token_ring.$i
    }
    

    .c create oval [expr $token_ring(x1)+$token_ring(delta)] [expr $token_ring(y1)+$token_ring(delta)] \
	           [expr $token_ring(x2)-$token_ring(delta)] [expr $token_ring(y2)-$token_ring(delta)] \
		  -fill $icon_color(empty)  -tag token_ring.mask
}

# according to the type to draw the appropriate color on the num'th 
proc display_token_ring_status {type num} {
    global icon_color

    .c itemconfigure token_ring.$num -fill $icon_color($type)
}


#
# Token Ring Status Window
#

proc show_token_ring_status_win { geometry } {
    set w .status_win
   
    toplevel $w
    wm geometry $w  400x200$geometry
    wm title $w "Status Window"

    frame $w.base -background Thistle
    pack $w.base -expand yes -fill both  

    frame  $w.frame1  -relief raised -bd 2
    frame  $w.frame2  -relief raised -bd 2
    frame  $w.frame3  -relief raised -bd 2
    frame  $w.frame4  -relief raised -bd 2
    frame  $w.frame5  -relief raised  -bd 2

    label  $w.src-label     -text "Source:"  -width 10  -justify left -relief raised
    label  $w.src           -text ""         -width 12  -justify left -relief ridge -fg red
    label  $w.dest-label    -text "Dest:"    -width 10  -justify left -relief raised
    label  $w.dest          -text ""         -width 12  -justify left -relief ridge -fg red
    label  $w.ppp-label     -text "PPP:"     -width 10  -justify left -relief raised
    label  $w.ppp           -text ""         -width 12  -justify left -relief ridge -fg red
    label  $w.rrr-label     -text "RRR:"     -width 10  -justify left -relief raised
    label  $w.rrr           -text ""         -width 12  -justify left -relief ridge -fg red
    label  $w.monitor-label -text "Monitor:" -width 10  -justify left -relief raised
    label  $w.monitor       -text ""         -width 45  -justify left -relief ridge -fg red
    label  $w.message-label -text "Mesage:"  -width 10  -justify left -relief raised
    text   $w.message -wrap char -width 20 -relief ridge 
    
    
    pack   $w.frame1  $w.frame2  $w.frame3  $w.frame4  -in $w.base -side top -fill both
    pack   $w.frame5 -in $w.base -side top -fill both -expand yes



     pack    $w.src-label  $w.src $w.dest-label  $w.dest \
 	    -in $w.frame1 -side left  -fill x -ipadx 3m
     pack    $w.ppp-label  $w.ppp $w.rrr-label   $w.rrr  \
 	    -in $w.frame2 -side left  -fill x -ipadx 3m
     pack    $w.monitor-label   $w.monitor   \
 	    -in $w.frame4 -side left  -fill x -ipadx 3m
     pack    $w.message-label  \
 	    -in $w.frame5 -side top  -fill x -ipadx 3m
     pack    $w.message -side top -fill both -expand true

    
}

proc show_info { name info } {
    set w .status_win
   
    $w.$name configure -text $info
    update
}

proc show_mesg { info } {
    set w .status_win
    
    $w.message insert end $info
    update
}

proc delete_mesg {  } {
    set w .status_win

    $w.message delete 1.0 end
    update
}


